#------------------------------------------------------------------------------
# Copyright 2008-2011 Istituto Nazionale di Fisica Nucleare (INFN)
#
# Licensed under the EUPL, Version 1.1 only (the "Licence").
# You may not use this work except in compliance with the Licence.
# You may obtain a copy of the Licence at:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Unless required by applicable law or agreed to in
# writing, software distributed under the Licence is
# distributed on an "AS IS" basis,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
# either express or implied.
# See the Licence for the specific language governing
# permissions and limitations under the Licence.
#------------------------------------------------------------------------------
"""
Part of WNoDeS framework.
Interface between WNoDeS processes an the batch system LSF
"""

#import time
from wnodes.utils import command
from wnodes.utils import batch_system

#import os
import sys
import threading
import commands

try:
    from xml.etree import ElementTree
except ImportError:
    try:
        from elementtree import ElementTree  # pylint: disable-msg=F0401
    except ImportError:
        sys.exit("package ElementTree is missing, exiting.")

class Badmin(command.Command):
    def __init__(self, profile, option, hostname):

        self.__profile__ = profile
        self.__hostname__ = hostname
        self.__option__ = option

    def _get_command(self):
        cmd = 'source %s;' % self.__profile__
        cmd += ' badmin %s' % self.__option__
        cmd += ' -C "WNoDeS admin action" %s' % self.__hostname__
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = {'0':{'hclose':'Close ...... done',
            'hopen':'Open ...... done'}
            }
        return fake_output

class Bjobs(command.Command):
    def __init__(self, profile, jobid = '', user = ''):
        
        self.__profile__ = profile
        self.__jobid__ = jobid
        self.__user__ = user

    def _get_command(self):
        cmd = 'source %s; bjobs -w' % self.__profile__
        if self.__user__ != '':
            cmd += ' -u %s' % self.__user__
        if self.__jobid__ != '':
            cmd += ' %s | grep %s' % (self.__jobid__, self.__jobid__)

        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = {0:'JOBID   USER    STAT  QUEUE      FROM_HOST   EXEC_HOST   JOB_NAME   SUBMIT_TIME\n4432916 superb036 RUN   superb     ce04-lcg    wn-205-01-15-02-b cr004_624025965 Mar  1 16:16',
            1:'No'
            }
        return fake_output

class Bkill(command.Command):
    def __init__(self, profile, jobid, user = ''):

        self.__profile__ = profile
        self.__jobid__ = jobid
        self.__user__ = user

    def _get_command(self):
        cmd = 'source %s;' % self.__profile__
        if self.__user__ != '':
            cmd += ' su -s /bin/bash -l %s -c' % self.__user__
        if self.__user__ != '':
            cmd += ' "bkill %s"' % self.__jobid__
        else:
            cmd += ' bkill %s' % self.__jobid__
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = 'Job <%s>: Job has already finished' % self.__jobid__
        return [0, fake_output]

class Bmod(command.Command):
    def __init__(self, profile, option, jobid):

        self.__profile__ = profile
        self.__option__ = option
        self.__jobid__ = jobid

    def _get_command(self):
        cmd = 'source %s;' % self.__profile__
        cmd += ' bmod %s %s' % (self.__option__, self.__jobid__)
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = 'Parameter of <%s> are being changed' % self.__jobid__
        return [0, fake_output]

class Breserve(command.Command):
    def __init__(self, profile, action, id, hostname = '', option = '', user = ''):

        self.__profile__ = profile
        self.__hostname__ = hostname
        self.__id_reservation__ = id
        self.__option__ = option
        self.__user__ = user
        self.__action__ = action
        self.type_reservation = {'add':'brsvadd',
            'del':'brsvdel'}

    def _get_command(self):
        cmd = 'source %s;' % self.__profile__
        cmd += ' %s' % self.type_reservation[self.__action__]
        if 'add' == self.__action__:
            cmd += ' -m %s %s' % (self.__hostname__, self.__option__)
            cmd += ' -N %s -u %s' % (self.__id_reservation__, self.__user__)
        else:
            cmd += ' %s' % self.__id_reservation__
        
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = ''
        return [0, fake_output]

class Brun(command.Command):
    def __init__(self, profile, hostname, jobid):

        self.__hostname__ = hostname
        self.__profile__ = profile
        self.__jobid__ = jobid

    def _get_command(self):
        cmd = 'source %s;' % self.__profile__
        cmd += ' brun -m %s %s' % (self.__hostname__, self.__jobid__)
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = 'Job <%s> is being forced to run.' % self.__jobid__
        return [0, fake_output]

class Bsub(command.Command):
    def __init__(self, profile, option, user = ''):

        self.__profile__ = profile
        self.__option__ = option
        self.__user__ = user

    def _get_command(self):
        cmd = 'source %s;' % self.__profile__
        if self.__user__ != '':
            cmd += ' su -s /bin/bash -l %s -c' % self.__user__
        if self.__user__ != '':
            cmd += ' "bsub < %s"' % self.__option__
        else:
            cmd += ' bsub < %s' % self.__option__
        #print cmd
        return cmd

    def get_fake_output(self):
        fake_output = 'Job <98798798> is submitted to default queue <argo>.'
        return [0, fake_output]

class LsfCommands(batch_system.BatchSystem):
    """ Set of lsf batch system commands"""
    def __init__(self, lsf_profile, expiration = 15):

        self.lsf_profile = lsf_profile
        self.expiration = expiration

        self.expiration_period = [0, 0]
        self.b_jobs_output = []
        self.jobs_info = {}
        self.host_list = {}

        #self.bait = "" 

    def badmin(self, option, hostname, fake = False):
        """badmin"""

        b_admin = Badmin(self.lsf_profile, option, hostname)
        b_admin_output = b_admin.get_output(fake = fake)

        if fake:
            fake_output = b_admin.get_fake_output()
            for cmd_status in fake_output.keys():
                for option_status in fake_output[cmd_status].keys():
                    if option_status == option:
                        b_admin_output.append(cmd_status)
                        b_admin_output.append(
                            fake_output[cmd_status][option_status]
                        )
                        break

        #print b_admin_output
        return b_admin_output

    def bjobs(self, jobid = '', user = '', fake = False, fake_status = 1):
        """bjobs"""

        b_jobs = Bjobs(self.lsf_profile, jobid = jobid, user = user)
        b_jobs_output = b_jobs.get_output(fake = fake)

        if fake:
            fake_output = b_jobs.get_fake_output()
            for cmd_status in fake_output.keys():
                if cmd_status == fake_status:
                    b_jobs_output.append(cmd_status)
                    b_jobs_output.append(
                        fake_output[cmd_status]
                    )
                    break

        #print b_jobs_output
        return b_jobs_output

    def bkill(self, jobid, user = '', fake = False):
        """bkill"""

        b_kill = Bkill(self.lsf_profile, jobid, user = user)
        b_kill_output = b_kill.get_output(fake = fake)

        if fake:
            b_kill_output = b_kill.get_fake_output()

        #print b_kill_output
        return b_kill_output

    def bmod(self, option, jobid, fake = False):
        """bmod"""

        b_mod = Bmod(self.lsf_profile, option, jobid)
        b_mod_output = b_mod.get_output(fake = fake)

        if fake:
            b_mod_output = b_mod.get_fake_output()

        #print b_mod_output
        return b_mod_output

    def breserve(self, action, id, hostname = '', option = '', user = '', fake = False):
        """breserve"""

        b_reserve = Breserve(self.lsf_profile, action, id, hostname = hostname,
            option = option, user = user)
        b_reserve_output = b_reserve.get_output(fake = fake)

        if fake:
            b_reserve_output = b_reserve.get_fake_output()

        #print b_reserve_output
        return b_reserve_output

    def brun(self, hostname, jobid, fake = False):
        """brun"""

        b_run = Brun(self.lsf_profile, hostname, jobid)
        b_run_output = b_run.get_output(fake = fake)

        if fake:
            b_run_output = b_run.get_fake_output()

        #print b_run_output
        return b_run_output

    def bsub(self, option, user = '', fake = False):
        """bsub"""

        #b_sub = Bsub(self.lsf_profile, option, reserved_host = self.bait,
        #    user = user)
        b_sub = Bsub(self.lsf_profile, option, user = user)
        b_sub_output = b_sub.get_output(fake = fake)

        if fake:
            b_sub_output = b_sub.get_fake_output()

        #print b_sub_output
        return b_sub_output

    def bkill(self, jobid, user = '', fake = False):
        """bkill"""

        b_kill = Bkill(self.lsf_profile, jobid, user = user)
        b_kill_output = b_kill.get_output(fake = fake)

        if fake:
            b_kill_output = b_kill.get_fake_output()

        #print b_kill_output
        return b_kill_output

        
